import cv2
import logging
import numpy as np
import sys
import torch
import torch.nn.functional as F

from collections import defaultdict
from IPython.display import display
from pathlib import Path
from time import time

# import some common detectron2 utilities
from detectron2.config import get_cfg
from detectron2.engine import DefaultPredictor

# Get ALDI imports
from aldi.config import add_aldi_config


class Predictor:

    def __init__(self, expt_cfg, model_weights, conf_thresh):

        '''Get Inference predictor for model with given config and weights'''
        cfg = get_cfg()
        cfg.set_new_allowed(True)
        add_aldi_config(cfg)
        cfg.merge_from_file(expt_cfg)
        cfg.MODEL.ROI_HEADS.SCORE_THRESH_TEST = 0.5  # set threshold for this model
        cfg.MODEL.WEIGHTS = model_weights
        self.predictor = DefaultPredictor(cfg)
        self.conf_thresh = conf_thresh


    def get_pred_bboxes(self, image_np):
        '''Need to rename this function. It gets predicted bounding boxes
           if confidence is high enough'''

        outputs = self.predictor(image_np)
        #print("outputs:", outputs)
    
        # Bbox predictions & scores
        pred_boxes = outputs['instances'].pred_boxes.tensor.cpu().numpy()
        pred_scores = outputs['instances'].scores.cpu().numpy()
    
        # Only accept bboxes with conf score above threshold
        pred_boxes_out = []
        for curr_pb, curr_score in zip(pred_boxes, pred_scores):
            if curr_score > self.conf_thresh:
                pred_boxes_out.append(curr_pb)
        return pred_boxes_out
